-- Listing 12.1 An Example of a Script
--
USE PUBS
SELECT au_id, au_lname FROM authors
SELECT pub_id, pub_name FROM publishers
INSERT publishers VALUES ('9997','SAMS Publishing', 'Seattle', 'WA','USA')
GO

SELECT * FROM stores
GO

DELETE publishers WHERE pub_id = '9997'
GO



-- In the following batch, each SQL statement is a separate transaction. 
-- Thus, this batch is actually three separate transactions. If any one of 
-- the statements fails, it doesn't affect the others. Each statement succeeds 
-- or fails on its own, regardless of the other statements in the batch.
-- 
INSERT table1 VALUES (1,'abcde')
UPDATE table1 SET col1 = 5 WHERE col1 = 1
DELETE FROM table1 WHERE col1 = 5
GO


-- Transactions can also provide performance benefits. By not writing 
-- the transaction log entries to disk until the transaction is completed, 
-- SQL Server can provide more efficient utilization of the disk. Therefore, 
-- it's beneficial to group statements. If you group the preceding statements 
-- into the transaction shown below, then only five log entries are written, 
-- instead of nine.
--
BEGIN TRAN
INSERT table1 VALUES (1,'abcde')
UPDATE table1 SET col1 = 5 WHERE col1 = 1
DELETE FROM table1 WHERE col1 = 5
COMMIT TRAN



-- For example, the following two code snippets are identical 
-- (in terms of transactions). Note that you can verify this usage 
-- by using the @@trancount variable.
--
CREATE TABLE table1 (col1 int not null)
BEGIN TRAN
INSERT table1 VALUES (1)
SELECT @@trancount
COMMIT TRAN


SET IMPLICIT_TRANSACTIONS ON
INSERT table1 values (2)
SELECT @@trancount
COMMIT TRAN



-- Listing 12.2 Stepping Through a Set of Transactions
--
BEGIN TRAN
    INSERT table1 values (1,'abcde')
    UPDATE table1 SET col1 = 5 WHERE col1 = 1
    DELETE FROM table1 WHERE col1 = 5
COMMIT TRAN



-- All that you can print is an ASCII string (string constant) 
-- or variable of type character (either fixed or variable length), as in this example:
--
PRINT "Hello"
PRINT @@version



-- The following example uses @msg to build one big string to print. 
-- You can't use concatenation operations and numeric variables in a 
-- PRINT statement, but you can use them in an assignment 
-- SET or SELECT and then print the result.
--
USE pubs
DECLARE @msg varchar(50),
@numWA tinyint
SELECT @numWA = COUNT(*) FROM stores
WHERE state = 'WA'
SELECT @msg = 'There are ' + convert(varchar(3), @numWA) +
' stores in Washington'

PRINT @msg



-- In the following example, if the average book price is greater 
-- than $15, you want to print one message; if the average book 
-- price is less than or equal to $15, you want to print a different message:
--
IF (SELECT avg(price) FROM titles) > $15
    PRINT 'Hold a big sale'
ELSE
    PRINT 'Time to raise prices'



-- The following example returns all the information about any 
-- book published by the publisher with ID 9933:
--
IF EXISTS
(SELECT * FROM titles WHERE pub_id = '9933')
BEGIN
    PRINT 'Here are the books: '
    SELECT * FROM titles WHERE pub_id = '9933'
END
ELSE
    PRINT 'No books from that publisher'



-- This query has the same results but asks the opposite question:
--
IF NOT EXISTS
(SELECT * FROM titles WHERE pub_id = '9933')
BEGIN
    PRINT 'No books from that publisher'
    RETURN
END
ELSE
BEGIN
    PRINT 'Here are the books'
    SELECT * FROM titles WHERE pub_id = '9933'
END



-- In the following example, for each row in the titles table, 
-- the value in the type column is compared to each value in the CASE list. 
-- The first matching value determines what expression will be returned. 
-- A searched case expression returns the expression associated with 
-- the first Boolean expression in the list that evaluates to true. 
-- If none of the expressions evaluate to true, the result expression 
-- after the ELSE is returned.
--
SELECT title_id type = CASE type
    WHEN 'business' THEN 'Business Book'
    WHEN 'psychology' THEN 'Psychology Book'
    WHEN 'mod_cook' THEN 'Modern Cooking Book'
    WHEN 'trad_cook' THEN 'Traditional Cooking Book'
    WHEN 'popular_comp' THEN 'Popular Computing Book'
    WHEN 'undecided' THEN 'No type determined yet'
    END
FROM titles



-- In this next example, for each row in the titles table, multiple 
-- expressions are evaluated. In this case, you're comparing the 
-- value of price against different values. The first expression that's 
-- true determines what expression will be returned.
--
SELECT title_id, cost = CASE
    WHEN price < 10 THEN 'Cheap'
    WHEN price BETWEEN 10 and 20 then 'Midrange'
    ELSE 'Expensive'
    END
FROM titles



-- Notice that the first CASE example was really just a special version 
-- of the searched CASE expression. You could have rewritten the first 
-- example as follows and gotten the same result:
--
SELECT title_id, type = case
    WHEN type = 'business' THEN 'Business Book'
    WHEN type = 'psychology' THEN 'Psychology Book'
    WHEN type = 'mod_cook' THEN 'Modern Cooking Book'
    WHEN type = 'trad_cook' THEN 'Traditional Cooking Book'
    WHEN type = 'popular_comp' THEN 'Popular Computing Book'
    ELSE 'No type determined yet'
    END
FROM titles



-- You could run the following code to create a sample table with some sample data:
--
USE pubs
GO
CREATE TABLE tblWages (
intEmpID int NOT NULL,
curHourly money NULL,
curSalary money NULL,
curCommission money NULL,
bigNumSales bigint NULL)
GO
INSERT tblWages VALUES (111, NULL, 52000, NULL, NULL)
INSERT tblWages VALUES (112, 14, NULL, NULL, NULL)
INSERT tblWages VALUES (113, NULL, NULL, 0.15, 50000)
INSERT tblWages VALUES (114, NULL, 73000, NULL, NULL)
INSERT tblWAGES VALUES (115, 4.90, NULL, NULL, NULL)
INSERT tblWages VALUES (116, NULL, 28500, NULL, NULL)
GO



-- After this table is created and loaded with data, you can determine 
-- the total amount paid to all employees by using the COALESCE function 
-- to receive only the non-NULL value found in the hourly, salary, and 
-- commission columns, as in this example:
--
SELECT intEmpID as "Employee ID", "Total Salary" = Convert(money,
(COALESCE(curHourly * 40 * 52, curSalary, curCommission * bigNumSales)))
FROM tblWages



-- The following example repeatedly checks the average price of all books. 
-- As long as that average is less than $20, the price of every book is updated. 
-- The repetition stops when the average price is greater than or equal to $20.
--
WHILE (SELECT AVG(price) FROM titles) < $20
BEGIN
    UPDATE titles SET price = price * 1.1
    PRINT 'all prices have been increased by 10%'
END



-- For example, to force an exclusive table lock on the authors table 
-- so that no one can modify the table while you examine it, you can run this query:
--
SELECT *
FROM authors (TABLOCKX)



-- A deadlock is a situation in which two transactions conflict with 
-- each other and the only resolution is to cancel one transaction. 
-- An example provides the easiest way to understand deadlocks:
-- Create the two tables shown here using the query analyzer, 
-- and populate them with data:
-- 
USE pubs
GO

CREATE TABLE tblChecking
(intAcctNum INT NOT NULL,
 strLastName CHAR(30) NOT NULL,
 curBalance MONEY NOT NULL
)
GO
CREATE TABLE tblSavings
(intAcctNum INT NOT NULL,
 strLastName CHAR(30) NOT NULL,
 curBalance MONEY NOT NULL
)
GO
INSERT tblChecking VALUES (1,'smith', $500.00)
INSERT tblChecking VALUES (2,'Jones', $300.00)
INSERT tblSavings VALUES (1,'smith', $100.00)
INSERT tblSavings VALUES (2,'Jones', $200.00)
GO



-- Open a second window in the Query Analyzer 
-- In the first window, run
use pubs
GO
BEGIN TRAN
    UPDATE tblChecking
    SET curBalance = curBalance + $100.00
    WHERE intAcctNum = 1



-- In the second window, run
-- 
USE pubs
GO
BEGIN TRAN
    UPDATE tblSavings
    SET curBalance = curBalance - $100.00
    WHERE intAcctNum = 2



-- Now, in the first window, run
-- UPDATE tblSavings
SET curBalance = curBalance - $100.00
WHERE intAcctNum = 1



-- In the second window, run
--
UPDATE tblChecking
SET curBalance = curBalance + $100.00
WHERE intAcctNum = 2



-- The original query is now complete. Run the following 
-- clean-up code in the first window:
--
COMMIT TRAN
GO
DROP TABLE tblChecking
DROP TABLE tblSavings
GO



-- To view which isolation level is in force, use the 
-- DBCC USEROPTIONS command. 
--
DBCC USEROPTIONS
GO



-- Notice the difference after turning on dirty reads with the 
-- READ UNCOMMITTED option
--
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
GO
DBCC USEROPTIONS
GO



